function MediumEditor(elements, options) {
    'use strict';
    return this.init(elements, options);
}

if (typeof module === 'object') {
    module.exports = MediumEditor;
}

(function (window, document) {
    'use strict';

    /* 黏贴文本 */
    function insertHtml(html) {
        var selection, range, el, fragment, node, lastNode, toReplace;

        if (document.queryCommandSupported('insertHTML')) {
            try {
                return document.execCommand('insertHTML', false, html);
            } catch (ignore) {}
        }

        selection = document.defaultView.getSelection();
        if (selection.getRangeAt && selection.rangeCount) {
            range = selection.getRangeAt(0);
            toReplace = range.commonAncestorContainer;
            // Ensure range covers maximum amount of nodes as possible
            // By moving up the DOM and selecting ancestors whose only child is the range
            if ((toReplace.nodeType === 3 && toReplace.nodeValue === range.toString()) ||
                    (toReplace.nodeType !== 3 && toReplace.innerHTML === range.toString())) {
                while (toReplace.parentNode &&
                        toReplace.parentNode.childNodes.length === 1 &&
                        !toReplace.parentNode.getAttribute('data-medium-element')) {
                    toReplace = toReplace.parentNode;
                }
                range.selectNode(toReplace);
            }
            range.deleteContents();

            el = document.createElement("div");
            el.innerHTML = html;
            fragment = document.createDocumentFragment();
            while (el.firstChild) {
                node = el.firstChild;
                lastNode = fragment.appendChild(node);
            }
            range.insertNode(fragment);

            // Preserve the selection:
            if (lastNode) {
                range = range.cloneRange();
                range.setStartAfter(lastNode);
                range.collapse(true);
                selection.removeAllRanges();
                selection.addRange(range);
            }
        }
    }

    /* 修复p中包含p */
    function fixPInP() {
        var $closeP = $('#js-closeP'),
            $openP = $('#js-openP'),
            $p = $closeP.closest('p');

        if ($p.length == 0) {
            $closeP.remove();
            $openP.remove();
            return;
        };

        $p.replaceWith($('<div>').append($p.clone()).html().replace(/<i.+?<\/i>/, '</p>').replace(/<i.+?<\/i>/, '<p>'));
    }

    function extend(b, a) {
        var prop;
        if (b === undefined) {
            return a;
        }
        for (prop in a) {
            if (a.hasOwnProperty(prop) && b.hasOwnProperty(prop) === false) {
                b[prop] = a[prop];
            }
        }
        return b;
    }

    function getParentsNodeType(node) {
        if (node.tagName) {
            if (node.tagName.toLowerCase() == 'b') {
                return getParentsNodeType(node.parentNode).concat(['b']);
            } else if(node.tagName.toLowerCase() == 'i'){
                return getParentsNodeType(node.parentNode).concat(['i']);
            } else if(node.tagName.toLowerCase() == 'u'){
                return getParentsNodeType(node.parentNode).concat(['u']);
            } else if(node.tagName.toLowerCase() == 'strike'){
                return getParentsNodeType(node.parentNode).concat(['s']);
            } else {
                return [];
            }
        } else {
            return getParentsNodeType(node.parentNode);
        }
    }

    function arrayIntersection(a, b) {
        var ai=0, bi=0;
        var result = new Array();
        while ( ai < a.length && bi < b.length )
        {
            if      ( a[ai] < b[bi] ) { ai++; }
            else if ( a[ai] > b[bi] ) { bi++; }
            else /* they're equal */
            {
                result.push ( a[ai] );
                ai++;
                bi++;
            }
        }
        return result;
    }

    // http://stackoverflow.com/questions/5605401/insert-link-in-contenteditable-element
    // by Tim Down
    function saveSelection() {
        var i,
            len,
            ranges,
            sel = window.getSelection();
        if (sel.getRangeAt && sel.rangeCount) {
            ranges = [];
            for (i = 0, len = sel.rangeCount; i < len; i += 1) {
                ranges.push(sel.getRangeAt(i));
            }
            return ranges;
        }
        return null;
    }

    function restoreSelection(savedSel) {
        var i,
            len,
            sel = window.getSelection();
        if (savedSel) {
            sel.removeAllRanges();
            for (i = 0, len = savedSel.length; i < len; i += 1) {
                sel.addRange(savedSel[i]);
            }
        }
    }

    // http://stackoverflow.com/questions/1197401/how-can-i-get-the-element-the-caret-is-in-with-javascript-when-using-contentedi
    // by You
    function getSelectionStart() {
        var node = document.getSelection().anchorNode,
            startNode = (node && node.nodeType === 3 ? node.parentNode : node);
        return startNode;
    }

    // http://stackoverflow.com/questions/4176923/html-of-selected-text
    // by Tim Down
    function getSelectionHtml() {
        var i,
            html = '',
            sel,
            len,
            container;
        if (window.getSelection !== undefined) {
            sel = window.getSelection();
            if (sel.rangeCount) {
                container = document.createElement('div');
                for (i = 0, len = sel.rangeCount; i < len; i += 1) {
                    container.appendChild(sel.getRangeAt(i).cloneContents());
                }
                html = container.innerHTML;
            }
        } else if (document.selection !== undefined) {
            if (document.selection.type === 'Text') {
                html = document.selection.createRange().htmlText;
            }
        }
        return html;
    }

    MediumEditor.prototype = {
        defaults: {
            allowMultiParagraphSelection: true,
            anchorInputPlaceholder: 'Paste or type a link',
            diffLeft: 0,
            diffTop: -10,
            forcePlainText: true,
            placeholder: '请输入正文，盗用作品会被关小黑屋嗷',
            targetBlank: false
        },


        init: function (elements, options) {
            this.elements = typeof elements === 'string' ? document.querySelectorAll(elements) : elements;
            if (this.elements.nodeType === 1) {
                this.elements = [this.elements];
            }
            if (this.elements.length === 0) {
                return;
            }
            this.isActive = true;
            this.parentElements = ['p', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'blockquote', 'pre'];
            this.id = document.querySelectorAll('.medium-editor-toolbar').length + 1;
            this.options = extend(options, this.defaults);
            return this.initElements()
                       .bindSelect()
                       .bindPaste()
                       .setPlaceholders()
                       .bindWindowActions()
        },

        initElements: function () {
            var i;
            for (i = 0; i < this.elements.length; i += 1) {
                this.elements[i].setAttribute('contentEditable', true);
                if (!this.elements[i].getAttribute('data-placeholder')) {
                    this.elements[i].setAttribute('data-placeholder', this.options.placeholder);
                }
                this.elements[i].setAttribute('data-medium-element', true);
                this.bindParagraphCreation(i).bindReturn(i).bindTab(i);
            }

            return this;
        },

        serialize: function () {
            var i,
                elementid,
                content = {};
            for (i = 0; i < this.elements.length; i += 1) {
                elementid = (this.elements[i].id !== '') ? this.elements[i].id : 'element-' + i;
                content[elementid] = {
                    value: this.elements[i].innerHTML.trim()
                };
            }
            return content;
        },

        bindParagraphCreation: function (index) {
            var self = this;
            this.elements[index].addEventListener('keyup', function (e) {
                var node = getSelectionStart(),
                    tagName;
                var el_block = 'p';
                if (node && node.getAttribute('data-medium-element') && node.children.length === 0 &&
                        !(self.options.disableReturn || node.getAttribute('data-disable-return'))) {
                    document.execCommand('formatBlock', false, el_block);
                }
                if (e.which === 13 && !e.shiftKey) {
                    node = getSelectionStart();
                    tagName = node.tagName.toLowerCase();
                    if (!(self.options.disableReturn || this.getAttribute('data-disable-return')) &&
                            tagName !== 'li' && !self.isListItemChild(node)) {

                        //patch by zzjin
                        node.removeAttribute('name');
                        document.execCommand('formatBlock', false, el_block);
                        if (tagName === 'a') {
                            document.execCommand('unlink', false, null);
                        }
                    }
                }
            });
            return this;
        },

        isListItemChild: function (node) {
            var parentNode = node.parentNode,
                tagName = parentNode.tagName.toLowerCase();
            while (this.parentElements.indexOf(tagName) === -1 && tagName !== 'div') {
                if (tagName === 'li') {
                    return true;
                }
                parentNode = parentNode.parentNode;
                if (parentNode && parentNode.tagName) {
                    tagName = parentNode.tagName.toLowerCase();
                } else {
                    return false;
                }
            }
            return false;
        },

        bindReturn: function (index) {
            var self = this;
            this.elements[index].addEventListener('keypress', function (e) {
                if (e.which === 13) {
                    // var node = getSelectionStart();
                    // if (node && node.innerText == '\n'){
                    //     e.preventDefault();
                    // }
                    if (self.options.disableReturn || this.getAttribute('data-disable-return')) {
                        e.preventDefault();
                    }
                }
            });
            return this;
        },

        bindTab: function (index) {
            this.elements[index].addEventListener('keydown', function (e) {
                if (e.which === 9) {
                    // Override tab only for pre nodes
                    var tag = getSelectionStart().tagName.toLowerCase();
                    if (tag === 'pre' || tag === 'p') {
                        e.preventDefault();
                        insertHtml('&nbsp;&nbsp;&nbsp;&nbsp;');
                    }
                }
            });
            return this;
        },

        bindSelect: function () {
            var self = this,
                timer = '',
                i;
            document.addEventListener('selectionchange', this.checkSelectionWrapper);
            document.getElementsByTagName("input")[0].addEventListener('keyup', function(){
                $(".titleCounts").text(this.value.length + "/30")
            });
            for (i = 0; i < this.elements.length; i += 1) {
                this.elements[i].addEventListener('keyup', this.checkSelectionWrapper);
                this.elements[i].addEventListener('blur', this.checkSelectionWrapper);
            }
            return this;
        },

        checkSelection: function () {
            var newSelection,
                selectionElement;
            return this;
        },
        checkSelectionWrapper: function () {
            var sel = getSelection(), resultStart, resultEnd, result, range = sel.getRangeAt(0);
            var textcount=getEditorCounts();
            if(textcount>0){
                 $('#editor').removeClass('medium-editor-placeholder');
            }else{
                 $('#editor').addClass('medium-editor-placeholder');
            }
            $(".editorCounts").text(textcount+ "字");
            getContent();
            if (sel.baseNode.nodeType == 1) return false;
            if (sel.rangeCount > 0) {
                if (range.startOffset == range.startContainer.length && range.startContainer.nextSibling && range.startContainer.nextSibling.textContent == range.endContainer.textContent) {
                    result = getParentsNodeType(range.endContainer).join(',');
                } else {
                    resultStart = getParentsNodeType(range.startContainer);
                    resultEnd = getParentsNodeType(range.endContainer);
                    result = arrayIntersection(resultStart, resultEnd).join(',');
                }
            }
            EditorBridge.getHtmlStatus(result);
        },

        checkSelectionElement: function (newSelection, selectionElement) {
            var i;
            this.selection = newSelection;
            this.selectionRange = this.selection.getRangeAt(0);
            for (i = 0; i < this.elements.length; i += 1) {
                if (this.elements[i] === selectionElement) {
                    this.setToolbarButtonStates()
                        .setToolbarPosition()
                        .showToolbarActions();
                    return;
                }
            }
            this.hideToolbarActions();
        },

        getSelectionElement: function () {
            var selection = window.getSelection(),
                range = selection.getRangeAt(0),
                current = range.commonAncestorContainer,
                parent = current.parentNode,
                result,
                getMediumElement = function(e) {
                    var parent = e;
                    try {
                        while (!parent.getAttribute('data-medium-element')) {
                            parent = parent.parentNode;
                        }
                    } catch (errb) {
                        return false;
                    }
                    return parent;
                };
            // First try on current node

            try {
                if (current.getAttribute('data-medium-element')) {
                    result = current;
                } else {
                    result = getMediumElement(parent);
                }
            // If not search in the parent nodes.
            } catch (err) {
                result = getMediumElement(parent);
            }
            return result;
        },


        // editorAnchorObserver: function(e) {
        //     var self = this,
        //         overAnchor = true,
        //         leaveAnchor = function() {
        //             // mark the anchor as no longer hovered, and stop listening
        //             overAnchor = false;
        //             self.activeAnchor.removeEventListener('mouseout', leaveAnchor);
        //         };

        //     if (e.target && e.target.tagName.toLowerCase() === 'a') {

        //         // Detect empty href attributes
        //         // The browser will make href="" or href="#top"
        //         // into absolute urls when accessed as e.targed.href, so check the html
        //         if (!/href=["']\S+["']/.test(e.target.outerHTML) || /href=["']#\S+["']/.test(e.target.outerHTML)) {
        //             return true;
        //         }

        //         this.activeAnchor = e.target;
        //         this.activeAnchor.addEventListener('mouseout', leaveAnchor);
        //         // show the anchor preview according to the configured delay
        //         // if the mouse has not left the anchor tag in that time
        //         setTimeout(function() {
        //             if (overAnchor) {
        //                 self.showAnchorPreview(e.target);
        //             }
        //         }, self.options.delay);

        //     }
        // },

        // bindAnchorPreview: function (index) {
        //     var self = this;
        //     this.elements[index].addEventListener('mouseover', function(e) {
        //         self.editorAnchorObserver(e);
        //     });
        //     return this;
        // },

        setTargetBlank: function () {
            var el = getSelectionStart(),
                i;
            if (el.tagName.toLowerCase() === 'a') {
                el.target = '_blank';
            } else {
                el = el.getElementsByTagName('a');
                for (i = 0; i < el.length; i += 1) {
                    el[i].target = '_blank';
                }
            }
        },

        bindWindowActions: function () {
            var timerResize,
                self = this;
            this.windowResizeHandler = function () {
                clearTimeout(timerResize);
                timerResize = setTimeout(function () {
                    //do nothing
                }, 100);
            };
            window.addEventListener('resize', this.windowResizeHandler);
            return this;
        },

        activate: function () {
            var i;
            if (this.isActive) {
                return;
            }

            if (this.toolbar !== undefined) {
                this.toolbar.style.display = 'block';
            }

            this.isActive = true;
            for (i = 0; i < this.elements.length; i += 1) {
                this.elements[i].setAttribute('contentEditable', true);
            }

            this.bindWindowActions()
                .bindSelect();
        },

        deactivate: function () {
            var i;
            if (!this.isActive) {
                return;
            }
            this.isActive = false;
            document.getElementById("editor").removeEventListener('touchend', this.checkSelectionWrapper);
            window.removeEventListener('resize', this.windowResizeHandler);

            for (i = 0; i < this.elements.length; i += 1) {
                this.elements[i].removeEventListener('keyup', this.checkSelectionWrapper);
                this.elements[i].removeEventListener('blur', this.checkSelectionWrapper);
                this.elements[i].removeEventListener('paste', this.pasteWrapper);
                this.elements[i].removeAttribute('contentEditable');
            }
        },

        bindPaste: function () {
            var i, self = this;
            this.pasteWrapper = function (e) {
                e.preventDefault();

                var paragraphs,
                    html = '',
                    p;

                this.classList.remove('medium-editor-placeholder');
                if (!self.options.forcePlainText) {
                    return this;
                }

                /* 获取剪切板的内容 */
                var pasteText = EditorBridge.getClipboardData();
                pasteText = $('<div>' + pasteText.replace(/<br\\{0,1}>/g, '\n') + '</div>').text();
                if (!self.options.disableReturn) {
                    paragraphs = pasteText.replace(/\r/g, '').split(/\n/g);
                    for (p = 0; p < paragraphs.length; p += 1) {
                        if (paragraphs[p] !== '') {
                            html += '<p>' + paragraphs[p] + '</p>';
                        } else {
                            html += '<p>' + '<br>' + '</p>';
                        }
                    }

                    insertHtml(html);
                } else {
                    insertHtml(pasteText);
                };
            };
            for (i = 0; i < this.elements.length; i += 1) {
                this.elements[i].addEventListener('paste', this.pasteWrapper);
            }
            return this;
        },

        setPlaceholders: function () {
            var i,
                activatePlaceholder = function (el) {
                    if (el.textContent.replace(/^\s+|\s+$/g, '') === '') {
                        el.classList.add('medium-editor-placeholder');
                    }
                },
                placeholderWrapper = function (e) {
                    this.classList.remove('medium-editor-placeholder');
                    if (e.type !== 'keydown') {
                        activatePlaceholder(this);
                    }
                };
            for (i = 0; i < this.elements.length; i += 1) {
                activatePlaceholder(this.elements[i]);
                this.elements[i].addEventListener('blur', placeholderWrapper);
                this.elements[i].addEventListener('keydown', placeholderWrapper);
            }
            return this;
        }

    };

}(window, document));
